// This test driver program reads an encoded record from a file 
// and decodes it and displays the results..

#include <stdio.h>
#include <stdlib.h>

#include "Class-definitions.h"
#include "Constant-definitions.h"
#include "InformationElements.h"
#include "Internode-definitions.h"
#include "PDU-definitions.h"

#define MAXMSGLEN 1024

int main (int argc, char** argv)
{
   OSOCTET	msgbuf[MAXMSGLEN];
   OSBOOL     aligned = FALSE, trace = TRUE, diag = FALSE;
   FILE*        fp;
   char*        filename = "message.dat";
   int		i, len, stat;

   // Process command line arguments 

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-a")) aligned = 1;
         else if (!strcmp (argv[i], "-u")) aligned = 0;
         else if (!strcmp (argv[i], "-v")) diag = TRUE;
         else if (!strcmp (argv[i], "-i")) filename = argv[++i];
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer -a | -u [ -v ] [ -o <filename>\n");
            printf ("   -a  use PER aligned encoding\n");
            printf ("   -u  use PER unaligned encoding\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -i <filename>  read encoded msg from <filename>\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   // Read input file into a memory buffer

   if (fp = fopen (filename, "rb")) {
      len = fread (msgbuf, 1, sizeof (msgbuf), fp);
      fclose (fp);
   }
   else {
      printf ("Error opening %s for read access\n", filename);
      return -1;
   }

   // Initialize decoder objects

   ASN1PERDecodeBuffer decodeBuffer (msgbuf, len, aligned);
   ASN1T_UL_DCCH_Message ulDCCH;
   ASN1C_UL_DCCH_Message ulDCCH_C (decodeBuffer, ulDCCH);

   decodeBuffer.SetTrace (trace);
   decodeBuffer.setDiag (diag);

   // Decode 

   stat = ulDCCH_C.Decode ();

   if (stat == 0) {
      if (trace) {
         printf ("decode successful:\n");
         printf ("Dump of decoded bit fields:\n");
         decodeBuffer.BinDump ("UL_DCCH_Message");
         printf ("\n");
         printf ("******\n");
         printf ("\n");
         ulDCCH_C.Print ("UL_DCCH_Message");
      }
   }
   else {
      printf ("decode failed\n");
      decodeBuffer.PrintErrorInfo ();
      return -1;
   }

   return 0;
}
