// This test driver program encodes a data record and writes the encoded 
// result to an output file..

#include <stdio.h>
#include <stdlib.h>
#include "Class-definitions.h"

#define MAXMSGLEN 1024

int main (int argc, char** argv)
{
   const OSOCTET* msgptr;
   OSOCTET	msgbuf[MAXMSGLEN];
   OSBOOL       aligned = FALSE, trace = TRUE, diag = FALSE;
   FILE*        fp;
   const char*  filename = "message.dat";
   int		i, len, stat, tvalue = 2;

   // Process command line arguments

   if (argc > 1) {
      for (i = 1; i < argc; i++) {
         if (!strcmp (argv[i], "-a")) aligned = 1;
         else if (!strcmp (argv[i], "-u")) aligned = 0;
         else if (!strcmp (argv[i], "-v")) diag = TRUE;
         else if (!strcmp (argv[i], "-o")) filename = argv[++i];
         else if (!strcmp (argv[i], "-t")) tvalue = atoi(argv[++i]);
         else if (!strcmp (argv[i], "-notrace")) trace = FALSE;
         else {
            printf ("usage: writer -a | -u [ -v ] [ -o <filename> ] ");
            printf ("[ -notrace ]\n");
            printf ("   -a  use PER aligned encoding\n");
            printf ("   -u  use PER unaligned encoding\n");
            printf ("   -v  verbose mode: print trace info\n");
            printf ("   -o <filename>  write encoded msg to <filename>\n");
            printf ("   -t  choice tag value to encode\n");
            printf ("   -notrace  do not display trace info\n");
            return 0;
         }
      }
   }

   // Populate message structure with some random data

   ASN1PEREncodeBuffer encodeBuffer (msgbuf, sizeof(msgbuf), aligned);
   encodeBuffer.setTrace (trace);
   encodeBuffer.setDiag (diag);

   ASN1T_UL_DCCH_Message msgData ; 
   ASN1C_UL_DCCH_Message UL_DCCH_Message (encodeBuffer, msgData); 

   OSCTXT* pctxt = UL_DCCH_Message.getCtxtPtr();
   ASN1T_UL_DCCH_Message* pvalue = &msgData;

   pvalue->m.integrityCheckInfoPresent = 1;
   pvalue->integrityCheckInfo.messageAuthenticationCode.numbits = 32;
   pvalue->integrityCheckInfo.messageAuthenticationCode.data[0] = 0xaa;
   pvalue->integrityCheckInfo.messageAuthenticationCode.data[1] = 0xbb;
   pvalue->integrityCheckInfo.messageAuthenticationCode.data[2] = 0xcc;
   pvalue->integrityCheckInfo.messageAuthenticationCode.data[3] = 0xdd;
   pvalue->integrityCheckInfo.rrc_MessageSequenceNumber = 13;

   pvalue->message.t =  
      T_UL_DCCH_MessageType_transportChannelReconfigurationComplete;

   pvalue->message.u.transportChannelReconfigurationComplete = 
      (ASN1T_TransportChannelReconfigurationComplete*) UL_DCCH_Message.
      memAlloc (sizeof(ASN1T_TransportChannelReconfigurationComplete));

   pvalue->message.u.transportChannelReconfigurationComplete->
      rrc_TransactionIdentifier = 2;

   pvalue->message.u.transportChannelReconfigurationComplete->
      m.ul_IntegProtActivationInfoPresent = 1;

   pvalue->message.u.transportChannelReconfigurationComplete->
      ul_IntegProtActivationInfo.rrc_MessageSequenceNumberList.n = 5;

   pvalue->message.u.transportChannelReconfigurationComplete->
      ul_IntegProtActivationInfo.rrc_MessageSequenceNumberList.elem[0] = 5;

   pvalue->message.u.transportChannelReconfigurationComplete->
      ul_IntegProtActivationInfo.rrc_MessageSequenceNumberList.elem[1] = 9;

   pvalue->message.u.transportChannelReconfigurationComplete->
      ul_IntegProtActivationInfo.rrc_MessageSequenceNumberList.elem[2] = 12;

   pvalue->message.u.transportChannelReconfigurationComplete->
      ul_IntegProtActivationInfo.rrc_MessageSequenceNumberList.elem[3] = 14;

   pvalue->message.u.transportChannelReconfigurationComplete->
      ul_IntegProtActivationInfo.rrc_MessageSequenceNumberList.elem[4] = 7;

   pvalue->message.u.transportChannelReconfigurationComplete->
      m.ul_TimingAdvancePresent = 1;

   pvalue->message.u.transportChannelReconfigurationComplete->
      ul_TimingAdvance = 3;

   pvalue->message.u.transportChannelReconfigurationComplete->
      m.count_C_ActivationTimePresent = 1;

   pvalue->message.u.transportChannelReconfigurationComplete->
      count_C_ActivationTime = 7;

   pvalue->message.u.transportChannelReconfigurationComplete->
      m.dummyPresent = 0;

   pvalue->message.u.transportChannelReconfigurationComplete->
      m.ul_CounterSynchronisationInfoPresent = 1;

   pvalue->message.u.transportChannelReconfigurationComplete->
      ul_CounterSynchronisationInfo.m.rB_WithPDCP_InfoListPresent = 1;

   {
      ASN1T_RB_WithPDCP_InfoList *plist = 
         &pvalue->message.u.transportChannelReconfigurationComplete->
         ul_CounterSynchronisationInfo.rB_WithPDCP_InfoList;

      ASN1T_RB_WithPDCP_Info* pdata0 =
         rtxMemAllocTypeZ (pctxt, ASN1T_RB_WithPDCP_Info);

      rtxDListInit(plist);
      pdata0->rb_Identity = 18;
      pdata0->pdcp_SN_Info = 1885;
      rtxDListAppend(pctxt, plist, pdata0);
   }
   {
      ASN1T_STARTList *plist = 
         &pvalue->message.u.transportChannelReconfigurationComplete->
         ul_CounterSynchronisationInfo.startList;

      ASN1T_STARTSingle* pdata0 = rtxMemAllocTypeZ (pctxt, ASN1T_STARTSingle);
      rtxDListInit(plist);
      pdata0->cn_DomainIdentity = CN_DomainIdentity::cs_domain;
      pdata0->start_Value.numbits = 20;
      pdata0->start_Value.data[0] = 0xFF;
      pdata0->start_Value.data[1] = 0xEE;
      pdata0->start_Value.data[2] = 0xDD;
      rtxDListAppend(pctxt, plist, pdata0);
   }

   pvalue->message.u.transportChannelReconfigurationComplete->
      m.laterNonCriticalExtensionsPresent = 1;

   pvalue->message.u.transportChannelReconfigurationComplete->
      laterNonCriticalExtensions.m.
      transportChannelReconfigurationComplete_r3_add_extPresent = 1;

   pvalue->message.u.transportChannelReconfigurationComplete->
      laterNonCriticalExtensions.
      transportChannelReconfigurationComplete_r3_add_ext.numbits = 19;

   {
      static OSOCTET bindata[] = { 0x12, 0x34, 0x56 } ;
      pvalue->message.u.transportChannelReconfigurationComplete->
         laterNonCriticalExtensions.
         transportChannelReconfigurationComplete_r3_add_ext.data = bindata;
   }

   // Encode

   encodeBuffer.SetTrace (trace);

   if ((stat = UL_DCCH_Message.Encode ()) == 0) {
      if (trace) {
         printf ("Encoding was successful\n");
         printf ("Hex dump of encoded record:\n");
         encodeBuffer.HexDump ();
         printf ("Binary dump:\n");
         encodeBuffer.BinDump ("UL_DCCH_Message");
      }
      msgptr = encodeBuffer.GetMsgPtr ();
      len = encodeBuffer.GetMsgLen ();
   }
   else {
      printf ("Encoding failed\n");
      encodeBuffer.PrintErrorInfo ();
      return -1;
   }
  
   // Write the encoded message out to the output file

   if (fp = fopen (filename, "wb")) {
      fwrite (msgptr, 1, len, fp);
      fclose (fp);
   }
   else {
      printf ("Error opening %s for write access\n", filename);
      return -1;
   }

   return 0;

}
